/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

%module Fragmentor
%{
#include "../../src/alignlib.h"

using namespace AlignLib;

#include "../../src/Fragmentor.h"
#include "../../src/HelpersFragmentor.h"
%}

class Fragmentor {
  /* class member functions-------------------------------------------------------------- */
 public:
    /* constructors and desctructors------------------------------------------------------- */
    
    /** destructor */
    virtual ~Fragmentor ();

    /** method for aligning two arbitrary objects */
    virtual FragmentVector * Fragment(const Alignandum *, const Alignandum *, Alignata *) = 0;		
    
};

/* 1. factory functions */

%new Fragmentor * makeFragmentorDiagonals(CALCTYPE gop, 
					  CALCTYPE gep, 
					  Alignator * dottor );
 

%new Fragmentor * makeFragmentorIterative( Alignata * dots, 
					   CALCTYPE min_score,
					   CALCTYPE gop,
					   CALCTYPE gep );

%new Fragmentor * makeFragmentorRepetitive( Alignator * alignator,
					    CALCTYPE min_score );

/* 3. convenience functions */
void writeFragments( std::ostream & output, const FragmentVector * fragments);

void deleteFragments( FragmentVector * fragments );

  /** rescore fragments. 
      The score of a fragment is the number of aligned residues plus gap penalties. 
      Gaps are penalized using affine gap penalties.
  */
void rescoreFragmentsNumberGaps( FragmentVector * fragments, 
				 CALCTYPE gop = 0, 
				 CALCTYPE gep = 0);

















