/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

%module Regularizor
%{
#include "../../src/alignlib.h"

using namespace AlignLib;

#include "../../src/Regularizor.h"
#include "../../src/HelpersRegularizor.h"
%}

class Regularizor {
 public:
    // constructors and desctructors

    /** destructor */
    virtual ~Regularizor ();
    
    /** copy the counts into the frequencies and regularize them by doing so. */
    virtual void fillFrequencies( TYPE_FREQUENCIES_COLUMN * frequencies, 
				  const TYPE_COUNTS_COLUMN * counts, 
				  const ALIGNANDUM_POSITION length ) const = 0;

};

/** factory functions */
%new Regularizor * makeNoRegularizor();

%new Regularizor * makeRegularizorDirichlet( TYPE_COUNTS fade_cutoff = 0);

%new Regularizor * makeRegularizorDirichletHash( TYPE_COUNTS fade_cutoff = 0);

%new Regularizor * makeRegularizorDirichletInterpolate( TYPE_COUNTS fade_cutoff = 0);

%new Regularizor * makeRegularizorDirichletPrecomputed( TYPE_COUNTS fade_cutoff = 0);

/** gets the default Regularizor object */ 
const Regularizor * getDefaultRegularizor();

/** sets the default Regularizor object */
const Regularizor * setDefaultRegularizor( const Regularizor * regularizor);

