/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

%module Alignata
%{
#include "../../src/alignlib.h"

using namespace AlignLib;

#include "../../src/Renderer.h"
#include "../../src/HelpersRenderer.h"

%}

/** protocoll class for aligned objects. 
	
    An alignment is a mapping.
    
    Each alignment should provide iterators. However,
    since the implementation differs between the different
    container types, I went for passive iterators, unlike
    the STL iterators, which are active. A passive iterator
    relies on the collection to manage its advancement,
    while 

    This class is a protocol class and as such defines only the general interface
    
    @author Andreas Heger
    @version $Id: Renderer.i,v 1.1.1.1 2002/07/01 17:15:20 heger Exp $
    @short protocol class for aligned objects
*/

class Renderer {


 public:
    // constructors and desctructors
    /** destructor */
    virtual ~Renderer ();

    // render one string into a different representation
    virtual std::string Render( const std::string & representation, 			      
				ALIGNANDUM_POSITION segment_start, 
				ALIGNANDUM_POSITION segment_end ) const = 0;


};

%new Renderer * makeRendererMView(const std::string & consensus);
