/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

%module SubstitutionMatrix
%{
#include "../../src/alignlib.h"

using namespace AlignLib;

#include "../../src/SubstitutionMatrix.h"
#include "../../src/HelpersSubstitutionMatrix.h"
%}

/**
   This class is an interface to substitution matrices. It exports a const pointer
   to a memory location to where the matrix stored. The class takes responsibility
   for reading, writing and manipulating matrices. The memory location does belong
   to the matrix it points to!
   
   Matrices are stored row-wise.

   A default matrix is supplied as the global libarary symbol DEFAULT_SUBSTMATRIX.

   @author Andreas Heger
   @version $Id: SubstitutionMatrix.i,v 1.1.1.1 2002/07/01 17:15:20 heger Exp $
   @short protocoll class for Substitution-matrices

*/

class SubstitutionMatrix {

 public:
    /* constructors and desctructors ----------------------------------------------- */
    /** desctructor */
    virtual ~SubstitutionMatrix ();

    /* accessors------------------------------------------------------------------------------ */
    /** return pointer to continuos location of matrix in memory */
    virtual const TYPE_MATRIX * getMatrix() const = 0;

    /** get number of rows of matrix */
    virtual int getNumRows() const = 0;
    
    /** get number of columns of matrix */
    virtual int getNumColums() const= 0;

    /** return the score between characters row and col */
    virtual TYPE_MATRIX getScore( RESIDUE row, RESIDUE col) const = 0;

    /** returns something, that can be used to access the member data. */
    virtual const SubstitutionMatrixData & getData() const = 0;

    /** write data to stream */
    virtual void Write(std::ostream & output) const = 0;
    
    /** clear memory */
    virtual void Clear() = 0;

};

/** factory functions */

/* create a Substitution Matrix given a memory location of log-odds scores */
%new SubstitutionMatrix * makeSubstitutionMatrixAA( TYPE_MATRIX_COLUMN_AA * matrix);

/** read a simple substitution matrix adn return it. Do not forget to tell the substitution matrix
  to clear its contents afterwards */
%new SubstitutionMatrix * readSubstitutionMatrixAA( const char * filename ); 

/** gets the default Regularizor object */ 
const SubstitutionMatrix * getDefaultSubstitutionMatrix();

/** sets the default SubstitutionMatrix object */
const SubstitutionMatrix * setDefaultSubstitutionMatrix( const SubstitutionMatrix * matrix);
 
