/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

%module Translator
%{
#include "../../src/alignlib.h"

using namespace AlignLib;

#include "../../src/Translator.h"
#include "../../src/HelpersTranslator.h"
%}

class Translator {
 public:
    // constructors and destructors

    /** destructor */
    virtual ~Translator () {};

    /** translate a string of residues from internal to real-world representation. A copy
	of the translated (null-terminated) string is returned.
	@param src		pointer to string of residues
	@param length	length of string
    */
    virtual char * Decode( const RESIDUE *src, int length) const = 0;
    
    /** translate a single residue from internal to real-world representation.
     */
    %name(DecodeResidue) virtual char Decode( const RESIDUE src) const = 0;
 
    /** translate at string of residues from real word presentation to internal representation. A
	copy of the translated string is returned.
	@param src		pointer to string of residues
	@param length	length of string
    */
    virtual RESIDUE * Encode( const char *, int ) const = 0;

    /** translate a single residue from real-world to internal representation.
     */
    %name(EncodeResidue) virtual RESIDUE Encode( const char) const = 0;
 
   /** check, if the supplied character is in the alphabet. */
    virtual bool isValidChar( const char ) const = 0;
    
    /** get code used for a masked character. */
    virtual RESIDUE getMaskCode() const = 0;

    /** get internal code used for a gap. */
    virtual RESIDUE getGapCode()  const = 0;
    
    /** get character used for a masked character. */
    virtual char getMaskChar() const = 0;

    /** get character used for a gap. */
    virtual char getGapChar()  const = 0;
    
};

/* -------------------------------------------------------------------------------------------------------------------- */
/* 1. factory functions */
%new Translator *  makeTranslatorSequence();
%new Translator *  makeTranslatorSequenceUC();

/* -------------------------------------------------------------------------------------------------------------------- */
/* 2. accessor functions for default objects */
/** return the library wide translator */
const Translator * getDefaultTranslator();

/** set the library wide translator and return last translator */
const Translator * setDefaultTranslator( const Translator * translator);

/* -------------------------------------------------------------------------------------------------------------------- */
/* 3. convenience functions */

