/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef ALIGNATOR_H
#define ALIGNATOR_H 1

#include "alignlib.h"

namespace AlignLib {

/**
   Base class for aligning two objects. The default call to use (with base class) is
   
   Alignator a;
   Alignata r;

   a( row, col, &r );

   The two objects to be aligned to each other are called row and col. This comes from
   the dynamic programming matrix where you have one sequence along the rows and the other 
   along the columns.

   This class is a protocol class and as such defines only the general interface
   
   @author Andreas Heger
   @version $Id: Alignator.h,v 1.1.1.1 2002/07/01 17:15:20 heger Exp $
   @short protocol class for aligned objects
*/ 

class Alignata;
class Alignandum;
class SubstitutionMatrix;

class Alignator {
  /* class member functions-------------------------------------------------------------- */
 public:
    /* constructors and desctructors------------------------------------------------------- */
    /** empty constructor */
    Alignator(); 
    
    /** destructor */
    virtual ~Alignator ();

    /** copy constructor */
    Alignator( const Alignator & src);
    
    /** method for aligning two arbitrary objects */
    virtual Alignata * Align(const Alignandum *, const Alignandum *, Alignata *) = 0;		

    /** align two objects and just return the score */
    virtual CALCTYPE getAlignmentScore(const Alignandum *, const Alignandum * ) = 0;		

    /** accessors */
    /** set gap opening penalty for row */
    virtual void setRowGop( CALCTYPE gop ) = 0;

    /** set gap extension penalty for row */
    virtual void setRowGep( CALCTYPE gep ) = 0;

    /** set gap opening penalty for col */
    virtual void setColGop( CALCTYPE gop ) = 0;

    /** set gap extension penalty for col */
    virtual void setColGep( CALCTYPE gep ) = 0;

    /** set substitution matrix to use */
    virtual void setSubstitutionMatrix( const SubstitutionMatrix * matrix ) = 0;
    
};

}

#endif /* ALIGNATOR_H */
