/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef HELPERS_ALIGNATOR_H
#define HELPERS_ALIGNATOR_H 1

#include "alignlib.h"

namespace AlignLib {
    
    /** Helper functions for class Alignata:
	
	1. factory functions
	
	2. accessor functions for default objects
	
	3. convenience functions
    */


class Alignator;
class Alignandum;
class Alignata;
class SubstitutionMatrix; 
class Fragmentor;

/* 1. factory functions */
 
    /** Perform full dynamic programming - Smith-Waterman */ 
    Alignator * makeFullDP( CALCTYPE gop, CALCTYPE gep, const SubstitutionMatrix * matrix = NULL);

    /** Perform full dynamic programming - Smith-Waterman with wrapping around */ 
    Alignator * makeAlignatorFullDPWrap( CALCTYPE gop, CALCTYPE gep, const SubstitutionMatrix * matrix = NULL);

    /** Perform full dynamic programming - Needleman-Wunsch */ 
    Alignator * makeAlignatorFullDPGlobal( CALCTYPE gop, CALCTYPE gep, 
					   bool penalize_left = true, 
					   bool penalize_right = true, 
					   const SubstitutionMatrix * matrix = NULL);

    /** make an alignator object, which aligns identical residues */
    Alignator * makeAlignatorIdentity();

    /** make an alignator object, which aligns identical residues */
    Alignator * makeAlignatorSimilarity(const SubstitutionMatrix * matrix = NULL);

    /** make an alignator object, which aligns similar tuples */
    Alignator * makeAlignatorTuples(int ktuple = 3, const SubstitutionMatrix * matrix = NULL);

    /** make an alignator object, which returns a dummy alignments */
    Alignator * makeAlignatorDummy( const Alignata * ali );

    /** make an alignator object, which returns a dummy alignment */
    Alignator * makeAlignatorPublishAlignata( Alignata * ali );
    
    /** make an alignator object, which does a dot-alignment with wrapping around. */
    Alignator * makeAlignatorDotsWrap(CALCTYPE gop, 
				      CALCTYPE gep, 
				      Alignator * alignator, 
				      const SubstitutionMatrix * matrix = NULL);

    /** make an alignator object, which does a dot-alignment with wrapping around. */
    Alignator * makeAlignatorDotsSquared(CALCTYPE gop, 
					 CALCTYPE gep, 
					 Alignator * alignator, 
					 const SubstitutionMatrix * matrix = NULL);

    /** make an alignator object, which does a dot-alignment with wrapping around. */
    Alignator * makeAlignatorDotsSquaredDiagonal(CALCTYPE gop, 
						 CALCTYPE gep, 
						 Alignator * alignator, 
						 CALCTYPE diagnal_gop = 0,
						 CALCTYPE diagonal_gep = 0,
						 const SubstitutionMatrix * matrix = NULL);

    /** make an alignator object, which aligns fragments. */
    Alignator * makeAlignatorFragmentsSquared(CALCTYPE gop, 
					      CALCTYPE gep, 
					      Fragmentor * fragmentor, 
					      const SubstitutionMatrix * matrix  = NULL);

    /* 2. accessor functions for default objects */
    

    /* 3. convenience functions */

    /** This function takes const arguments, which is consistend with the Align-
	method of alignator-objects. However, the alignandum-objects
	have to be first copied into non-const objects before aligning. This costs
	time.
     */
Alignata * performIterativeAlignment( Alignata * dest, 
				      const Alignandum * src_1, 
				      const Alignandum * src_2, 
				      Alignator * a, 
				      CALCTYPE min_score); 

/** This function takes non-const arguments. No copying is necessary and 
    it should work fine.
*/ 
Alignata * performIterativeAlignment( Alignata * dest, 
				      Alignandum * src_1, 
				      Alignandum * src_2, 
				      Alignator * a, 
				      CALCTYPE min_score); 


}

#endif	/* HELPERS_ALIGNATOR_H */
