/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <iostream>
#include <iomanip>

#include "ImplAlignandum.h"
#include "Translator.h"
#include "HelpersTranslator.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

namespace AlignLib {

//--------------------------------------------------------------------------------------
ImplAlignandum::ImplAlignandum() : 
  mFrom(0), mTo(0), mLength(0), mIsPrepared(false) {
}    

//--------------------------------------------------------------------------------------
ImplAlignandum::~ImplAlignandum () {
#ifdef DEBUG
    if (verbose > LL3) cout << "ImplAlignandum::~ImplAlignandum()" << endl;
#endif
}

//--------------------------------------------------------------------------------------
ImplAlignandum::ImplAlignandum(const ImplAlignandum & src) {
#ifdef DEBUG
    if (verbose > LL3) cout << "ImplAlignandum::ImplAlignandum( const ImplAlignandum &)" << endl;
#endif

    mFrom   = src.mFrom;
    mTo     = src.mTo;
    mLength = src.mLength;
    mIsPrepared = src.mIsPrepared;

}

//--------------------------------------------------------------------------------------
void ImplAlignandum::Mask( ALIGNANDUM_POSITION from, ALIGNANDUM_POSITION to) {
  ALIGNANDUM_POSITION j;
  for (j = from; j <= to; j++) 
    Mask( j );
}

//--------------------------------------------------------------------------------------
ALIGNANDUM_POSITION ImplAlignandum::getLength() const {
  return (mTo - mFrom + 1);
}

//--------------------------------------------------------------------------------------
void ImplAlignandum::useFullLength() {
  mFrom = 1;
  mTo   = mLength;
}

//--------------------------------------------------------------------------------------
void ImplAlignandum::useSegment(ALIGNANDUM_POSITION from, ALIGNANDUM_POSITION to) {
  mFrom = from;
  mTo   = to;
  if (mTo > mLength)
    mTo = mLength;
}

//--------------------------------------------------------------------------------------
ALIGNANDUM_POSITION ImplAlignandum::getFrom() const {
  return mFrom;
}

//--------------------------------------------------------------------------------------
ALIGNANDUM_POSITION ImplAlignandum::getTo() const {
   return mTo;
}

//--------------------------------------------------------------------------------------
ALIGNANDUM_POSITION ImplAlignandum::getOffset( ALIGNANDUM_POSITION pos) const {
  return (pos + mFrom - 1);
}

//--------------------------------------------------------------------------------------
void ImplAlignandum::setTrueLength( ALIGNANDUM_POSITION length) const {
  mLength = length;
}

//--------------------------------------------------------------------------------------
ALIGNANDUM_POSITION ImplAlignandum::getTrueLength() const {
  return mLength;
}

//--------------------------------------------------------------------------------------
bool ImplAlignandum::isPrepared() const {
    return mIsPrepared;
}

//--------------------------------------------------------------------------------------
void ImplAlignandum::setPrepared( bool flag ) const {
    mIsPrepared = flag;
} 

//--------------------------------------------------------------------------------------
char ImplAlignandum::asChar( ALIGNANDUM_POSITION pos ) const {
  return getDefaultTranslator()->Decode( asResidue( pos ));
}


//--------------------------------------------------------------------------------------
// use faster implementations in subclasses, if you prefer
std::string ImplAlignandum::asString() const {
  std::string ret_val("");

  for (ALIGNANDUM_POSITION i = 1; i <= getLength(); i++) 
    ret_val += getDefaultTranslator()->Decode( asResidue(i) );
  
  return ret_val;
}
   
  
} // namespace AlignLib
