/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef IMPL_ALIGNATA_MATRIX_DIAGONAL_H
#define IMPL_ALIGNATA_MATRIX_DIAGONAL_H 1

#include <iosfwd>
#include "alignlib.h"
#include "ImplAlignataMatrix.h"

namespace AlignLib {

    /** class for aligned objects. 

	if alignment is empty, mRowFrom is larger than mRowTo. This property is needed
	at some checks (for example in ImplAlignataMatrix_iterator)

	Residues are kept in a list in memory. Note, that in this form of alignments,
	several column residues can be aligned to the same row residue and vice versa.

	Think of this Alignment as a dot-matrix. Actually, this is what it is used for
	in AlignatorDot. However, the full matrix is not stored memory, but as 
	a list of aligned residues (sorted first by row, then by column). The class provides
	and index for quick mapping of row to column (always the smallest col is returned).

	Note: some of the functions meaningful in simple pairwise alignment object have
	a consistent but slightly different sense here. For example the alignment length
	returns the number of dots might therefore be longer than the sequences. 

	This subclass ImplAlignataMatrix keeps the dots sorted by diagonal. Therefore
	the iteration procedure is different.

	@author Andreas Heger
	@version $Id: ImplAlignataMatrixDiagonal.h,v 1.1.1.1 2002/07/01 17:15:20 heger Exp $
	@short protocol class for aligned objects
    */



class Alignandum;

class ImplAlignataMatrixDiagonal : public ImplAlignataMatrix {

    friend class Iterator_const;
    
 public:

    //------------------> constructors / destructors <---------------------------------------------------------
    /** constructor */
    ImplAlignataMatrixDiagonal( long ndots = 0);
    
    /** copy constructor */
    ImplAlignataMatrixDiagonal( const ImplAlignataMatrixDiagonal &src );

    /** destructor */
    virtual ~ImplAlignataMatrixDiagonal();

    //------------------------------------------------------------------------------------------------------------
    virtual ImplAlignataMatrixDiagonal * getNew() const;
    
    /** return an identical copy */
    virtual ImplAlignataMatrixDiagonal * getClone() const;
    
    //----------------> accessors <------------------------------------------------------------------------------

    /** maps a residue from row to column. returns 0, if not found. This is quick, since there is 
	only one lookup in the array needed.*/
    virtual ALIGNANDUM_POSITION mapRowToCol( ALIGNANDUM_POSITION pos ) const;

 protected:
    /** build index */
    virtual void buildIndex() const;

    /** sort Dots by row and col */
    void sortDots() const;
    
 private:
    /** number of diagonals in alignemnt box */
    mutable ALIGNANDUM_POSITION mNumDiagonals;
    
};

						  

}

#endif /* _ALIGNATA_H */

