/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <iostream>
#include "alignlib.h"

#include "Alignator.h"

#include "Alignata.h"
#include "HelpersAlignata.h"

#include "Alignandum.h"
#include "AlignException.h"

#include "SubstitutionMatrix.h"

#include "ImplAlignator.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

namespace AlignLib {


//-------------------------------------------------------------------------------------------------------------------------------------------------
  
ImplAlignator::ImplAlignator( const SubstitutionMatrix * subst_matrix,
			      CALCTYPE row_gop, CALCTYPE row_gep, 
			      CALCTYPE col_gop, CALCTYPE col_gep) :
  mRowGop(row_gop), mRowGep( row_gep), mColGop( col_gop), mColGep( col_gep) {
  
    mSubstitutionMatrix = subst_matrix;
    
    if (mColGop == 0) {
	mColGop = mRowGop;
	mColGep = mRowGep;
    }
}

ImplAlignator::~ImplAlignator() {
#ifdef DEBUG
    if (verbose > LL3) cout << "ImplAlignator::~ImplAlignator()" << endl;
#endif

}

ImplAlignator::ImplAlignator( const ImplAlignator & src ) : 
  mRowGop( src.mRowGop), mRowGep( src.mRowGep), mColGop( src.mColGop), mColGep( src.mColGep) {

  mSubstitutionMatrix = src.mSubstitutionMatrix;
}
	
//-------------------------------------------------------------------------------------------------------------------------------
void ImplAlignator::setRowGop( CALCTYPE gop ) { mRowGop = gop;} 
void ImplAlignator::setRowGep( CALCTYPE gep ) { mRowGep = gep;}
void ImplAlignator::setColGop( CALCTYPE gop ) { mColGop = gop;}
void ImplAlignator::setColGep( CALCTYPE gep ) {mColGep = gep; }
void ImplAlignator::setSubstitutionMatrix( const SubstitutionMatrix * matrix ) { mSubstitutionMatrix = matrix; }

CALCTYPE ImplAlignator::getRowGop() { return mRowGop; }
CALCTYPE ImplAlignator::getRowGep() { return mRowGep; }
CALCTYPE ImplAlignator::getColGop() { return mColGop; }
CALCTYPE ImplAlignator::getColGep() { return mColGep; }
const SubstitutionMatrix * ImplAlignator::getSubstitutionMatrix() { return mSubstitutionMatrix; }

ALIGNANDUM_POSITION ImplAlignator::getRowLength() { return mRowLength; }
ALIGNANDUM_POSITION ImplAlignator::getColLength() { return mColLength; }


void ImplAlignator::startUp( const Alignandum * row, const Alignandum * col, Alignata * ali) {
#ifdef DEBUG
    if (verbose > LL3) cout << "ImplAlignator::startUp(const Alignandum *, const Alignandum *, Alignata *) " << endl;
#endif
    row->Prepare();
    col->Prepare();
    
    mRowLength = row->getLength();
    mColLength = col->getLength();
  
    ali->Clear();
  
}

void ImplAlignator::cleanUp(const Alignandum * row, const Alignandum *col, Alignata * ali) {
#ifdef DEBUG
    if (verbose > LL3) cout << "ImplAlignator::cleanUp(const Alignandum *, const Alignandum *, Alignata *) " << endl;
#endif
  
  // correct the alignment for offsets
  ALIGNANDUM_POSITION row_offset =  row->getFrom() - 1;
  ALIGNANDUM_POSITION col_offset =  col->getFrom() - 1;
  
  if (row_offset > 0 || col_offset > 0)
    ali->moveAlignment( row_offset, col_offset);
  
}

CALCTYPE ImplAlignator::getAlignmentScore(const Alignandum * row, const Alignandum *col ) {

  // call Align() with a dummy alignment
  Alignata * ali = makeAlignataSet();
  Align( row, col, ali);
  CALCTYPE score = ali->getScore();
  delete ali;

  return score;

}

} // namespace AlignLib
