/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <iostream>
#include <iomanip>
#include <math.h>
#include "alignlib.h"
#include "Alignandum.h"

#include "Alignata.h"
#include "HelpersAlignata.h"

#include "SubstitutionMatrix.h"
#include "HelpersSubstitutionMatrix.h"

#include "ImplAlignatorDummy.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

namespace AlignLib {

Alignator * makeAlignatorDummy( const Alignata * ali) {
  return new ImplAlignatorDummy( ali );
}

//---------------------------------------------------------< constructors and destructors >--------------------------------------
ImplAlignatorDummy::ImplAlignatorDummy ( const Alignata * ali) : ImplAlignator( getDefaultSubstitutionMatrix(), 0, 0 ), mAlignata ( ali ) {
}
		       
ImplAlignatorDummy::~ImplAlignatorDummy () {
}

ImplAlignatorDummy::ImplAlignatorDummy (const ImplAlignatorDummy & src ) : ImplAlignator(src) {
}

Alignata * ImplAlignatorDummy::Align( const Alignandum * row, const Alignandum * col, Alignata * result) {

  startUp(row, col, result );

  // copy alignment only in region given and move to (1,1)
  copyAlignata( result, mAlignata, 
		row->getFrom(), row->getTo(), 
		col->getFrom(), col->getTo() );

  result->moveAlignment( 1 - row->getFrom() , 1 - col->getFrom() );

  // cout << "mapped\n" << *result << endl;

  // Do not call cleanup, as it moves the alignment!
  // cleanUp(row, col, result );

  return result;
}


} // namespace AlignLib
