/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <iostream>
#include <iomanip>
#include <math.h>
#include "alignlib.h"
#include "Alignandum.h"
#include "Alignata.h"

#include "SubstitutionMatrix.h"
#include "HelpersSubstitutionMatrix.h"

#include "ImplAlignatorIdentity.h"
#include "Translator.h"
#include "HelpersTranslator.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

namespace AlignLib {


//---------------------------------------------------------< constructors and destructors >--------------------------------------
ImplAlignatorIdentity::ImplAlignatorIdentity () : ImplAlignator( getDefaultSubstitutionMatrix(), 0, 0 ) {
}
		       
ImplAlignatorIdentity::~ImplAlignatorIdentity () {
}

ImplAlignatorIdentity::ImplAlignatorIdentity (const ImplAlignatorIdentity & src ) : ImplAlignator(src) {
}

Alignata * ImplAlignatorIdentity::Align( const Alignandum * row, const Alignandum * col, Alignata * result) {
#ifdef DEBUG
    if (verbose > LL3) cout << "ImplAlignator::Align(const Alignandum *, const Alignandum *, Alignata *) " << endl;
#endif

  startUp(row, col, result );

  ALIGNANDUM_POSITION i, j;
  ALIGNANDUM_POSITION row_length = getRowLength();
  ALIGNANDUM_POSITION col_length = getColLength();

  CALCTYPE total_score = 0;
  
  RESIDUE mask_code = getDefaultTranslator()->getMaskCode();

  for (i = 1; i <= row_length; i++) {
    if (row->asResidue(i) != mask_code) {
      for (j = 1; j <= col_length; j++) 
	if (row->asResidue(i) == col->asResidue(j)) {
	  result->addPair( new RESIDUEPAIR(i,j,1) );
	  total_score += 1;
	}
    }
  }

  result->setScore( total_score );

  cleanUp(row, col, result );

  return result;
}


} // namespace AlignLib
