/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef IMPL_FRAGMENTOR_H
#define IMPL_FRAGMENTOR_H 1

#include "alignlib.h"
#include "Fragmentor.h"
#include "Alignata.h"

namespace AlignLib {

/**
   Base class for aligning two objects. The default call to use (with base class) is
   
   Fragmentor a;
   Alignata r;

   a( row, col, &r );

   The two objects to be aligned to each other are called row and col. This comes from
   the dynamic programming matrix where you have one sequence along the rows and the other 
   along the columns.

   This class is a protocol class and as such defines only the general interface
   
   @author Andreas Heger
   @version $Id: ImplFragmentor.h,v 1.1.1.1 2002/07/01 17:15:20 heger Exp $
   @short protocol class for aligned objects
*/ 

class SubstitutionMatrix;
class Alignandum;
class Alignata;

class ImplFragmentor : public Fragmentor {
  /* class member functions-------------------------------------------------------------- */
 public:
    /* constructors and desctructors------------------------------------------------------- */
    /** constructor */
    ImplFragmentor();

    /** destructor */
    virtual ~ImplFragmentor ();

    /** copy constructor */
    ImplFragmentor( const ImplFragmentor & src);
    
    virtual FragmentVector * Fragment(const Alignandum *, const Alignandum *, Alignata *);		
 protected:
    /** get length of row object */
    ALIGNANDUM_POSITION getRowLength();

    /** get length of col */
    ALIGNANDUM_POSITION getColLength();

    /** perform initialisation before alignment. Overload, but call this function in subclasses! */
    virtual void startUp( const Alignandum * row, const Alignandum * col, Alignata * ali);
    
    /** perform cleanup after alignment */
    virtual void cleanUp(const Alignandum * row, const Alignandum * col, Alignata * ali);                     

    /* perform the actual alignment */
    virtual void performFragmentation( const Alignandum * row, const Alignandum * col, const Alignata * sample) = 0;

    /** pointer to substitution matrix to use. not private, so children can access it */
    const SubstitutionMatrix * mSubstitutionMatrix;

 protected:
    /** length of object in row */
    ALIGNANDUM_POSITION mRowLength;
    /** length of object in col */
    ALIGNANDUM_POSITION mColLength;

    /** result */
    FragmentVector * mFragments;
};

}

#endif /* FRAGMENTOR_H */



