/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef IMPL_FRAGMENTOR_ITERATIVE_H
#define IMPL_FRAGMENTOR_ITERATIVE_H 1

#include "alignlib.h"
#include "ImplFragmentor.h"
#include "Alignata.h"

namespace AlignLib {

/**
   Base class for aligning two objects. The default call to use (with base class) is
   
   Fragmentor a;
   Alignata r;

   a( row, col, &r );

   The two objects to be aligned to each other are called row and col. This comes from
   the dynamic programming matrix where you have one sequence along the rows and the other 
   along the columns.

   This class is a protocol class and as such defines only the general interface
   
   @author Andreas Heger
   @version $Id: ImplFragmentorIterative.h,v 1.1.1.1 2002/07/01 17:15:20 heger Exp $
   @short protocol class for aligned objects
*/ 

class SubstitutionMatrix;
class Alignandum;
class Alignata;

class ImplFragmentorIterative : public ImplFragmentor {
  /* class member functions-------------------------------------------------------------- */
 public:
    /* constructors and desctructors------------------------------------------------------- */
    /** constructor */
    ImplFragmentorIterative( Alignata * dots, 
			     CALCTYPE min_score, 
			     CALCTYPE gop,
			     CALCTYPE gep);

    /** destructor */
    virtual ~ImplFragmentorIterative ();

    /** copy constructor */
    ImplFragmentorIterative( const ImplFragmentorIterative & src);

 protected:
    /** the alignator used to create dot-plots */
    Alignata *mDots;

    /** minimum score for alignment */
    CALCTYPE mMinScore;

    /** gap penalties */
    CALCTYPE mGop;
    CALCTYPE mGep;

    /** perform the actual alignment */
    virtual void performFragmentation( const Alignandum * row, 
				       const Alignandum * col, 
				       const Alignata * sample);
};

}

#endif /* FRAGMENTOR_H */
