/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <iostream>
#include "alignlib.h"

#include "Fragmentor.h"

#include "Alignata.h"
#include "HelpersAlignata.h"

#include "AlignataIterator.h"

#include "Alignandum.h"
#include "AlignException.h"

#include "SubstitutionMatrix.h"
#include "HelpersSubstitutionMatrix.h"

#include "Alignator.h"
#include "HelpersAlignator.h"

#include "ImplFragmentorRepetitive.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

namespace AlignLib {

/*---------------------factory functions ---------------------------------- */

  /** make an alignator object, which does a dot-alignment. The default version can be given an AlignataMatrix-
      object */
Fragmentor * makeFragmentorRepetitive( Alignator * alignator, CALCTYPE min_score ) {
  return new ImplFragmentorRepetitive( alignator, min_score );
}


//----------------------------------------------------------------------------------------------------
  
ImplFragmentorRepetitive::ImplFragmentorRepetitive( Alignator * alignator, 
						    CALCTYPE min_score ):

  ImplFragmentor(),
  mAlignator( alignator ),
  mMinScore( min_score) {
}


ImplFragmentorRepetitive::~ImplFragmentorRepetitive() {
#ifdef DEBUG
    if (verbose > LL3) cout << "ImplFragmentorRepetitive::~ImplFragmentorRepetitive()" << endl;
#endif
}

ImplFragmentorRepetitive::ImplFragmentorRepetitive( const ImplFragmentorRepetitive & src ) : 
    ImplFragmentor(src), 
    mAlignator(src.mAlignator), 
    mMinScore( src.mMinScore ) {
}

//------------------------------------------------------------------------------------------------
void ImplFragmentorRepetitive::performFragmentation( const Alignandum * row, 
						     const Alignandum * col, 
						     const Alignata * sample) {
    
  /* since src1 and src2 are const, I have to create two work-copies, 
     so that the boundaries can be changed. */

  Alignandum * copy_row = row->getClone();
  Alignandum * copy_col = col->getClone();
  
  while ( 1 ) {
    
    Alignata * result = sample->getNew();
    mAlignator->Align( copy_row, copy_col, result);
    if (result->getScore() >= mMinScore) {

      mFragments->push_back( result );    
      copy_row->Mask( result->getRowFrom(), result->getRowTo() );
      copy_col->Mask( result->getColFrom(), result->getColTo() );
      
    } else {
      delete result;    
      break;      
    }
  }

  delete copy_row;
  delete copy_col;

}
  

} // namespace AlignLib




