/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <iostream>
#include "Regularizor.h"
#include "HelpersRegularizor.h"
#include "ImplNoRegularizor.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

namespace AlignLib {

  /** factory functions */
  Regularizor * makeNoRegularizor() { return new ImplNoRegularizor();}


//---------------------------------------------------------< constructors and destructors >--------------------------------------
ImplNoRegularizor::ImplNoRegularizor () {
}
		       
ImplNoRegularizor::~ImplNoRegularizor () {
}

ImplNoRegularizor::ImplNoRegularizor (const ImplNoRegularizor & src ) {
}

//--------------------------------------------------------------------------------------------------------------------------------
void ImplNoRegularizor::fillFrequencies( TYPE_FREQUENCIES_COLUMN * frequencies, 
					 const TYPE_COUNTS_COLUMN * counts, 
					 const ALIGNANDUM_POSITION length ) const {
#ifdef DEBUG
  if (verbose > LL3) cout << "void ImplNoRegularizor::fillFrequencies( TYPE_FREQUENCIES_COLUMN *,const TYPE_COUNTS_COLUMN *, const ALIGNANDUM_POSITION length ) const" << endl;
#endif

  // simply calculate frequencies
 
  ALIGNANDUM_POSITION column;
  TYPE_COUNTS ntotal;
  int i;

  for (column = 1; column <= length; column++) {
    ntotal = 0;
 
    for (i = 0; i < PROFILEWIDTH; i ++)
      ntotal += counts[column][i];

    if (ntotal == 0)
      ntotal = 1;
    
    for (i = 0; i < PROFILEWIDTH; i++) {
      TYPE_FREQUENCIES f = (TYPE_FREQUENCIES)((TYPE_FREQUENCIES)counts[column][i] / (TYPE_FREQUENCIES)ntotal);
      frequencies[column][i] = f;
    }
    
  }                    

}

} // namespace AlignLib
