/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef IMPL_REGULARIZOR_DIRICHLET_H
#define IMPL_REGULARIZOR_DIRICHLET_H 1

#include "alignlib.h"
#include "Regularizor.h"


namespace AlignLib {

#define NCOMPONENTS 9

    typedef double TYPE_WA_COLUMN[NCOMPONENTS];
    typedef double TYPE_A_COLUMN[PROFILEWIDTH];
    typedef double TYPE_BETA_DIFFERENCES[NCOMPONENTS];

  /** Implementation of a class that regularizes count columns based
      on the 9-component Dirichlet-mixture by Kimmen Sjoerlander.
      
      @author Andreas Heger
      @version $Id: ImplRegularizorDirichlet.h,v 1.1.1.1 2002/07/01 17:15:20 heger Exp $
      @short protocol class for sequence weighters
      
  */


class ImplRegularizorDirichlet : public Regularizor {
 public:
    // constructors and desctructors

    /** default constructor */
    ImplRegularizorDirichlet  ( TYPE_COUNTS fade_cutoff );
    
    /** copy constructor */
    ImplRegularizorDirichlet  (const ImplRegularizorDirichlet &);

    /** destructor */
    virtual ~ImplRegularizorDirichlet ();
    
    /** copy the counts into the frequencies and regularize them by doing so. */
    virtual void fillFrequencies( TYPE_FREQUENCIES_COLUMN * frequencies, 
				  const TYPE_COUNTS_COLUMN * counts, 
				  const ALIGNANDUM_POSITION length ) const;

 private:

    /** the cutoff used for fading out the Dirichlet-mixture */
    TYPE_COUNTS mFadeCutoff;

 protected:
    /** helper array, contains the |a_j|. Static, because I need only one for all regularizers. 
	declaring the array puts it not in the heap. Can not be private, because derived classes
	need to access it. The same applies to mA.
    */
    TYPE_WA_COLUMN mWa;

    /** pointer to the mixture components */
    TYPE_A_COLUMN * mA;

    /** helper array, contains lgamma(|a_j]) */
    double * mLGamma_Wa;
    
    /** helper array, contains sum_i(lgamma(a_ji) */
    double * mSumLGamma_Ai;

    /** This function encapsulates that part of the algorithm, that needs to access the lgamma-function. It
	has been externalized, so that it can be overloaded to implement different speed-ups.
	
	It returns the maximum difference.

	The implemention here uses the lgamma-function directly and is therefore quite slow.
    */
    virtual double calculateBetaDifferences(  TYPE_BETA_DIFFERENCES beta_differences, const TYPE_COUNTS * n, TYPE_COUNTS ntotal ) const;
    
    /** fill one frequencies column with the Dirichlet-Mixture */
    virtual void fillColumn( TYPE_FREQUENCIES * frequencies, 
			     TYPE_BETA_DIFFERENCES beta_differences, 
			     const TYPE_COUNTS * n, 
			     TYPE_COUNTS ntotal ) const;
    
};


}

#endif /* IMPL_REGULARIZOR_DIRICHLET_H */

