/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <iostream>
#include <fstream>
#include <iomanip>
#include <stdlib.h>
#include "alignlib.h"
#include "AlignException.h"
#include "ImplSubstitutionMatrix.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

namespace AlignLib {

//---------------------------------------------------------< constructors and destructors >--------------------------------------
ImplSubstitutionMatrix::ImplSubstitutionMatrix( TYPE_MATRIX * matrix, int num_rows, int num_cols, bool this_own) :
    mNumRows(num_rows), mNumCols(num_cols), mMatrix(matrix), mThisOwn( this_own) {
    mData.mNumCols = mNumCols;
    mData.mNumRows = mNumRows;
    mData.mMatrixPointer = mMatrix;
}

ImplSubstitutionMatrix::~ImplSubstitutionMatrix () {
  if (mThisOwn) 
    Clear();
}
  
ImplSubstitutionMatrix::ImplSubstitutionMatrix (const ImplSubstitutionMatrix & src ) {
#ifdef DEBUG
  if (verbose > LL3) cout << "ImplSubstitutionMatrix::ImplSubstitutionMatrix( const ImplSubstitutionMatrix & src )" << endl;
#endif
}
//--------------------------------------------------------------------------------------------------------------------------------

const TYPE_MATRIX * ImplSubstitutionMatrix::getMatrix() const { return mMatrix; }
int ImplSubstitutionMatrix::getNumRows() const { return mNumRows; }
int ImplSubstitutionMatrix::getNumColums() const { return mNumCols; } 
const ImplSubstitutionMatrixData & ImplSubstitutionMatrix::getData() const { return mData;}

//--------------------------------------------------------------------------------------------------------------------------------
TYPE_MATRIX ImplSubstitutionMatrix::getScore( RESIDUE row, RESIDUE col) const {
  return mMatrix[row * mNumCols + col];
}

//--------------------------------------------------------------------------------------------------------------------------------
void ImplSubstitutionMatrix::Clear() {
  delete [] mMatrix;
}

//--------------------------------------------------------------------------------------------------------------------------------
void ImplSubstitutionMatrix::Write(std::ostream & output) const {
    int i, j;
    for (i = 0; i < mNumRows; i++) {
	cout << "Row " << i << " ";
	for (j = 0; j < mNumCols; j++) {
	    cout << setw(5) << getScore( i, j );
	}
	cout << "\n";
    }
}


} // namespace AlignLib













