/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <iostream>
#include <fstream>
#include <iomanip>
#include <stdlib.h>
#include "alignlib.h"
#include "AlignException.h"
#include "ImplSubstitutionMatrixAA.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

namespace AlignLib {


//------------------------------------------------------------------------------------------------------------
/** defintion of global object */

//------------> factory functions <------------------------------------------------------------------------------------------------
SubstitutionMatrix * makeSubstitutionMatrixAA( TYPE_MATRIX_COLUMN_AA * matrix, bool this_own) {
    return new ImplSubstitutionMatrixAA( matrix, this_own );
}

//----------------------------------------------------------------------------------------------------------------------------
SubstitutionMatrix * readSubstitutionMatrixAA( const char * filename ) {
    // simple reading routine for sustitution matrix
  
  ifstream fin( filename);  
  if (!fin)       
    throw AlignException("Could not open file in readSubstitutionMatrixAA");
  
  TYPE_MATRIX_COLUMN_AA * matrix = new TYPE_MATRIX_COLUMN_AA[MATRIXWIDTH_AA];
  
  TYPE_MATRIX * p = (TYPE_MATRIX*)matrix;
  int max_iterations = MATRIXWIDTH_AA * MATRIXWIDTH_AA;
  
  while (max_iterations > 0 && !fin.eof()) {                          // while (fin) does not work!!       
    fin >> *p;
    p++;
    max_iterations--;
  }
    
  if (max_iterations != 0 )
    throw AlignException("Read incomplete matrixin readSubstitionMatrixAA");

  fin.close();  
  
  return new ImplSubstitutionMatrixAA( matrix, true );
}; 


//---------------------------------------------------------< constructors and destructors >--------------------------------------
ImplSubstitutionMatrixAA::ImplSubstitutionMatrixAA( TYPE_MATRIX_COLUMN_AA * matrix, bool this_own ) :
    ImplSubstitutionMatrix( (TYPE_MATRIX*)matrix, MATRIXWIDTH_AA, MATRIXWIDTH_AA, this_own) {

    mMatrixAA = matrix;
    mData.mNumRows = MATRIXWIDTH_AA;
    mData.mNumCols = MATRIXWIDTH_AA;
    mData.mAAMatrixPointer = matrix;
}

ImplSubstitutionMatrixAA::~ImplSubstitutionMatrixAA () {
}
  
ImplSubstitutionMatrixAA::ImplSubstitutionMatrixAA (const ImplSubstitutionMatrixAA & src ) : ImplSubstitutionMatrix( src) {
}

//--------------------------------------------------------------------------------------------------------------------------------
const ImplSubstitutionMatrixAAData & ImplSubstitutionMatrixAA::getData() const { return mData;}

//--------------------------------------------------------------------------------------------------------------------------------
TYPE_MATRIX ImplSubstitutionMatrixAA::getScore( RESIDUE row, RESIDUE col) const {
    return mMatrixAA[row][col];
}


} // namespace AlignLib
