/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <math.h>

#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <algorithm>

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

#include "alignlib.h"
#include "Alignator.h"
#include "Alignandum.h"
#include "Statistics.h"

using namespace std;

namespace AlignLib {

/** factory functions */
NormalDistributionParameters * makeNormalDistributionParameters() { 
  return new NormalDistributionParameters; 
}

//---------------------------------------------------------------
CALCTYPE * fillScoresVector( CALCTYPE * dest,
			     const Alignandum * row, 
			     const Alignandum * col, 
			     Alignator * alignator,
			     unsigned int n_iterations,
			     unsigned int n_iterations_shuffle = 1,
			     ALIGNANDUM_POSITION window_size = 0) {

  Alignandum * clone = row->getClone();
  
  for (unsigned int i = 0; i < n_iterations; i++) {
    clone->Shuffle( n_iterations_shuffle, window_size );
    dest[i] = alignator->getAlignmentScore( clone, col );
  }
  
  delete clone;
  
  return dest;
}


/*---------------> other functions <----------------------------- */
void calculateZScoreParameters( NormalDistributionParameters * result,
				const Alignandum * row, 
				const Alignandum * col, 
				Alignator * alignator,
				unsigned int n_iterations,
				unsigned int n_iterations_shuffle,
				ALIGNANDUM_POSITION window_size) {

  unsigned int i;
  double total;
  double m, s;

  CALCTYPE * scores = new CALCTYPE[n_iterations];
  fillScoresVector( scores, row, col, alignator, n_iterations, n_iterations_shuffle, window_size );
    
  /* calculate mean */
  total = 0; for (i = 0; i < n_iterations; i++) total+= (double)scores[i];
  m = total / (double)n_iterations;
    
  /* calculate standard-deviation */
  total = 0; for (i = 0; i < n_iterations; i++) total+= ((double)scores[i] - m) * ((double)scores[i] - m);
  s = sqrt( total / (double)n_iterations);
    
  result->mMean = m;
  result->mStandardDeviation = s;
  
  delete [] scores;
}
   
/*-------------------------------------------------------------------------*/    
/** iteratively align src1 and src2 and fit an extreme value 
      distribution. The fit is done using linear regression.
*/
void calculateEVDParameters( EVDParameters * result,
			     const Alignandum * row, 
			     const Alignandum * col, 
			     Alignator * alignator,
			     unsigned int n_iterations,
			     unsigned int n_iterations_shuffle,
			     ALIGNANDUM_POSITION window_size) {

  // 1. get scores
  CALCTYPE * scores = new CALCTYPE[n_iterations];
  fillScoresVector( scores, row, col, alignator, n_iterations );

  // 2. build histogram
  
}

  

    
} // namespace AlignLib






