/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/** Test alignata objects
    
    note: for AlignatorIdentity, AlignatorSimilarity, etc. the 
    alignments look terrible, since they are wraparound pairwise
    alignments.

*/

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <fstream>
#include <strstream>
#include <vector>

#include <time.h> 

#include "alignlib.h"

#include "SubstitutionMatrix.h"
#include "HelpersSubstitutionMatrix.h"

#include "HelpersSequence.h"

#include "HelpersProfile.h"

#include "Alignator.h"
#include "HelpersAlignator.h"

#include "Alignata.h"
#include "HelpersAlignata.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace AlignLib;

// typedef enum MODE { PAIR, MATRIX };



bool TestPairwiseAlignment(AlignLib::Alignator * a, 
			   AlignLib::Alignata * ali,
			   const Alignandum * row,
			   const Alignandum * col,
			   const char * result_row,
			   const char * result_col,
			   CALCTYPE score ) {
  
  a->Align( row, col, ali);
  
  std::string new_row;
  std::string new_col;
  std::string r_row(result_row);
  std::string r_col(result_col);
  
  writeAlignataCompressed( ali, new_row, new_col );
    
  if (ali->getScore() == score &&
      new_row == r_row &&
      new_col == r_col )
    return true;
  else {
    cout << "new_row=" << new_row << " new_col=" << new_col << " r_row=" << r_row << " r_col=" << r_col << endl;
    cout << *ali;
    return false;
  }
  
}

/* test pairwise alignments. For the calculation of the score it is assumed, 
   that the identity matrix is used (match = 1, mismatch = -1, gop = -0.2, gep = -0.1).
*/
void TestPairAlignator( Alignator * a,
			Alignata * ali) {

  { 
    cout << "testing...Align( seq1, seq2 )#1...";

    Alignandum * a1 = makeSequence( "AAAAACCCCCAAAAA");
    Alignandum * a2 = makeSequence( "AAAAACCCCCAAAAA");
    
    if (
	TestPairwiseAlignment( a, ali, a1, a2, "+15", "+15", 15) &&
	TestPairwiseAlignment( a, ali, a2, a1, "+15", "+15", 15) 
	) 
      cout << "passed" << endl;
    else 
      cout << "failed" << endl;

    delete a1;
    delete a2;
  }

  { 
    cout << "testing...Align( seq1, seq2 )#2...";

    Alignandum * a1 = makeSequence( "AAAAACCCCCAAAAA");
    Alignandum * a2 = makeSequence( "CCCCC");
    
    if (
	TestPairwiseAlignment( a, ali, a1, a2, "+5", "+5", 5) &&
	TestPairwiseAlignment( a, ali, a2, a1, "+5", "+5", 5) 
	) 
      cout << "passed" << endl;
    else 
      cout << "failed" << endl;

    delete a1;
    delete a2;
  }

  { 
    cout << "testing...Align( seq1, seq2 )#3...";

    Alignandum * a1 = makeSequence( "AAAAACCCAACCCAAAAA");
    Alignandum * a2 = makeSequence( "CCCCCC");
    
    if (
	TestPairwiseAlignment( a, ali, a1, a2, "+8", "+3-2+3", 4.8) &&
	TestPairwiseAlignment( a, ali, a2, a1, "+3-2+3", "+8", 4.8) 
	) 
      cout << "passed" << endl;
    else 
      cout << "failed" << endl;

    delete a1;
    delete a2;
  }

  { 
    cout << "testing...Align( seq1, seq2 )#4...";

    Alignandum * a1 = makeSequence( "AAAAACCCACCCAAAAA");
    Alignandum * a2 = makeSequence( "CCCKCCC");
    
    if (
	TestPairwiseAlignment( a, ali, a1, a2, "+7", "+7", 5) &&
	TestPairwiseAlignment( a, ali, a2, a1, "+7", "+7", 5) 
	) 
      cout << "passed" << endl;
    else 
      cout << "failed" << endl;

    delete a1;
    delete a2;
  }
  


//   //--------------------------------------------------------
//   seq1 = makeSequence( "AAAAACCCCCAAAAA");
//   seq2 = makeSequence( "AAAAAKKKKKAAAAA");
//   ali = a->Align( seq1, seq2, ali);
//   cout << *seq1 << endl << *seq2 << endl << *ali;
//   cout << "Score: " << ali->getScore() << " (should be: 37 = 10 * 4 - 2 * 1 - 10 *  0.1)" <<endl;
//   if (mode == PAIR) writePairAlignment( cout, seq1, seq2, ali);
//   delete seq1;
//   delete seq2;
  
//   //--------------------------------------------------------
//   seq1 = makeSequence( "AAAACAAAAAA" );
//   seq2 = makeSequence( "AAAAAAAAAA");
//   ali = a->Align( seq1, seq2, ali);
//   cout << *seq1 << endl << *seq2 << endl << *ali;
//   cout << "Score: " << ali->getScore() << " (should be: 37 = 10 * 4 - 2 * 1 - 10 *  0.1)" <<endl;
//   if (mode == PAIR) writePairAlignment( cout, seq1, seq2, ali);
//   delete seq1;
//   delete seq2;

//   //--------------------------------------------------------
//   seq1 = makeSequence( "AAAAKKKKAAAAAA" );
//   seq2 = makeSequence( "KKAKKKGGG");
//   ali = a->Align( seq1, seq2, ali);
//   cout << *seq1 << endl << *seq2 << endl << *ali;
//   cout << "Score: " << ali->getScore() << " (should be: 19 = 1 * 4 + 3 * 5)" <<endl;
//   if (mode == PAIR) writePairAlignment( cout, seq1, seq2, ali);
//   delete seq1;
//   delete seq2;

//   //--------------------------------------------------------
//   // check wraparound-alignment
//   seq1 = makeSequence( "YKKAAAAAAYKKAAAAAAYKKAAAAAAYKKAAAAAA" );
//   seq2 = makeSequence( "KKAAAAAA");
//   ali = a->Align( seq1, seq2, ali);
//   cout << *seq1 << endl << *seq2 << endl << *ali;
//   ali = a->Align( seq2, seq1, ali);
//   cout << *ali;
//   delete seq1;
//   delete seq2;

  //--------------------------------------------------------
  cout << "Test finished..." << endl;
  
}

int main () {

  SubstitutionMatrix * matrix = makeSubstitutionMatrixAAIdentity();
  
  delete setDefaultSubstitutionMatrix( matrix );

  {
    cout << "---------------------Testing AlignatorFullDP----------------------------------" << endl;
    
    Alignator * a = makeFullDP( -1.0, -0.1 );
    Alignata * ali= makeAlignataVector();
    TestPairAlignator( a, ali );
    delete a;
    delete ali;
  }


//   cout << "---------------------Testing AlignatorFullDPWrap----------------------------------" << endl;
//   a1 = makeAlignatorFullDPWrap( -1, -0.1 );
//   ali= makeAlignataSet();
//   TestAlignator( a1, ali, PAIR   );
//   delete a1;
//   delete ali;

//    cout << "---------------------Testing AlignatorFullDPGlobal: end-gap-penalties on both sides----------------------------------" << endl;
//    a1 = makeAlignatorFullDPGlobal( -1, -0.1, true, true );
//    ali= makeAlignataSet();
//    TestAlignator( a1, ali, PAIR );
//    delete a1;
//    delete ali;

//    cout << "---------------------Testing AlignatorFullDPGlobal: no right end-gap-penalty----------------------------------" << endl;
//    a1 = makeAlignatorFullDPGlobal( -1, -0.1, true, false );
//    ali= makeAlignataSet();
//    TestAlignator( a1, ali, PAIR );
//    delete a1;
//    delete ali;

//    cout << "---------------------Testing AlignatorFullDPGlobal: no left end gap-penalty----------------------------------" << endl;
//    a1 = makeAlignatorFullDPGlobal( -1, -0.1, false, true );
//    ali= makeAlignataSet();
//    TestAlignator( a1, ali, PAIR );
//    delete a1;
//    delete ali;

//    cout << "---------------------Testing AlignatorFullDPGlobal: no end gap-penalty----------------------------------" << endl;
//    a1 = makeAlignatorFullDPGlobal( -1, -0.1, false, false );
//    ali= makeAlignataSet();
//    TestAlignator( a1, ali, PAIR );
//    delete a1;
//    delete ali;


//   cout << "---------------------Testing AlignatorIdentity----------------------------------" << endl;
//   a1 = makeAlignatorIdentity();
//   ali= makeAlignataMatrixRow();
//   TestAlignator( a1, ali, PAIR );
//   delete a1;
//   delete ali;
  
//   cout << "---------------------Testing AlignatorSimilarity----------------------------------" << endl;
//   a1 = makeAlignatorSimilarity();		
//   ali= makeAlignataMatrixRow();
//   TestAlignator( a1, ali, MATRIX);
//   delete a1;
//   delete ali;

//   cout << "---------------------Testing AlignatorTuples----------------------------------" << endl;
//   a1 = makeAlignatorTuples();		// this uses default values for ktuple-size and substitution matrix
//   ali= makeAlignataMatrixRow();
//   TestAlignator( a1, ali, MATRIX );
//   delete a1;
//   delete ali;

  {
    cout << "---------------------Testing AlignatorDots----------------------------------" << endl;

    // this uses default values for ktuple-size and substitution matrix
    Alignator * t  = makeAlignatorTuples( 1 );
    Alignator * a  = makeAlignatorDotsSquared( -1, -0.1, t);		
    Alignata  * ali= makeAlignataVector();

    TestPairAlignator( a, ali );
    delete a;
    delete t;
    delete ali;
  }

  delete matrix;
  
}
