/*
  alignlib - a library for aligning protein sequences

  $Id$

  Copyright (C) 2004 Andreas Heger
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/** Test the MultipleAlignment - object
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <fstream>

#include "Alignandum.h"
#include "HelpersSequence.h"
#include "HelpersProfile.h"
#include "Alignator.h"
#include "HelpersAlignator.h"
#include "Alignatum.h"
#include "HelpersAlignatum.h"
#include "Alignata.h"
#include "HelpersAlignata.h"
#include "MultipleAlignment.h"
#include "HelpersMultipleAlignment.h"

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

using namespace std;

const char FILE_SEQ[] = "data/test.seq";
const char FILE_FASTA[] = "data/test.fasta";


int main () {

  {
    // create a multiple alignment 
    AlignLib::MultipleAlignment * m1 = AlignLib::makeMultipleAlignment();
    m1->Add(AlignLib::makeAlignatumFromString("-AAAAA-CCAAA-"));
    m1->Add(AlignLib::makeAlignatumFromString("AAAAAA-CCAAAA"));
    m1->Add(AlignLib::makeAlignatumFromString("-A-AAA-CCA-A-"));
    m1->Add(AlignLib::makeAlignatumFromString("AAAAAAA--AAAA"));
    
    cout << *m1 << endl;
    
    // write segments of the alignment and check numbering
    m1->Write( cout, 0, 13 ); cout << endl;
    m1->Write( cout, 1, 12 ); cout << endl;
    m1->Write( cout, 2, 11 ); cout << endl;
    m1->Write( cout, 3, 10 ); cout << endl;

    // add two multiple alignments
    AlignLib::MultipleAlignment * m2 = AlignLib::makeMultipleAlignment();
    
    m2->Add( m1 );
    m2->Add( m1 );
    
    cout << *m2 << endl;
    
    delete m1;
    delete m2;
    
    // add two multiple alignments using an alignment between them:
    
    m1 = AlignLib::makeMultipleAlignment();
    m1->Add(AlignLib::makeAlignatumFromString("123456789"));
    m1->Add(AlignLib::makeAlignatumFromString("123456789"));
    m1->Add(AlignLib::makeAlignatumFromString("123456789"));
    
    AlignLib::Alignata * ali = AlignLib::makeAlignataSet();
    
    ali->addPair( new AlignLib::RESIDUEPAIR( 2,2, 1));
    ali->addPair( new AlignLib::RESIDUEPAIR( 3,3, 1));
    ali->addPair( new AlignLib::RESIDUEPAIR( 5,4, 1));
    ali->addPair( new AlignLib::RESIDUEPAIR( 6,5, 1));
    ali->addPair( new AlignLib::RESIDUEPAIR( 7,7, 1));
    ali->addPair( new AlignLib::RESIDUEPAIR( 8,8, 1));
    
    cout << *ali << endl;
    
    m2 = AlignLib::makeMultipleAlignment();
    m2 -> Add( m1 );
    cout << *m2 << endl;
    
    m2->Add( m1, ali, true );
    cout << *m2 << endl;
    
    m2->Add( m1, ali, false);
    cout << *m2 << endl;
    
    delete m1;
    delete m2;
    //-------------------------add new objects using an aligment */
    
    m1 = AlignLib::makeMultipleAlignment();
    m1->Add(AlignLib::makeAlignatumFromString("123456789"));
    m1->Add(AlignLib::makeAlignatumFromString("123456789"));
    m1->Add(AlignLib::makeAlignatumFromString("123456789"));
    
    m1->Add(AlignLib::makeAlignatumFromString("123456789"), ali, true );
    cout << *m1 << endl;
    
    m1->Add(AlignLib::makeAlignatumFromString("123456789"), ali, false );
    cout << *m1 << endl;
    
    delete ali;
    delete m1;
    
  }

  // check mali dots
  std::cout << "## checking MultipleAlignmentDots" << std::endl;
  {

    // create a multiple alignment 
    AlignLib::MultipleAlignment * m1 = AlignLib::makeMultipleAlignmentDots( false );
    AlignLib::MultipleAlignment * m2 = AlignLib::makeMultipleAlignmentDots( false );    
    
    AlignLib::Alignata * a1 = AlignLib::makeAlignataVector();
    AlignLib::fillAlignataIdentity( a1, 2, 4, 0 );
    AlignLib::fillAlignataIdentity( a1, 7, 8, 0 );

    AlignLib::Alignata * a2 = AlignLib::makeAlignataVector();
    AlignLib::fillAlignataIdentity( a2, 1, 5, +1 );

    AlignLib::Alignata * a3 = AlignLib::makeAlignataVector();
    AlignLib::fillAlignataIdentity( a3, 2, 6, -1 );
    AlignLib::fillAlignataIdentity( a3, 7, 7, 0 );
    AlignLib::fillAlignataIdentity( a3, 8, 8, +1 );
    
    m1->Add(AlignLib::makeAlignatumFromString("ABCDGHJL"), a1);
    m1->Add(AlignLib::makeAlignatumFromString(".ABCDEFGH"), a2);
    m1->Add(AlignLib::makeAlignatumFromString("BCDEFIJKLM"), a3);

    m2->Add(AlignLib::makeAlignatumFromString("ABCDGHJL"), a1);
    m2->Add(AlignLib::makeAlignatumFromString(".ABCDEFGH"), a2);
    m2->Add(AlignLib::makeAlignatumFromString("BCDEFIJKLM"), a3);
    
    cout << *m1 << endl;
    cout << *m2 << endl;    

    AlignLib::Alignata * aa1 = AlignLib::makeAlignataVector();
    AlignLib::fillAlignataIdentity( aa1, 2, 9, 0 );    

    m1->Add( m2, aa1 );

    cout << *m1 << endl;        
    delete a1;
    delete a2;
    delete a3;
    delete m1;

  }
  
  {

    // create a multiple alignment 
    AlignLib::MultipleAlignment * m1 = AlignLib::makeMultipleAlignment();
    AlignLib::MultipleAlignment * m2 = AlignLib::makeMultipleAlignmentDots( false );    
    
    AlignLib::Alignata * a1 = AlignLib::makeAlignataVector();
    AlignLib::fillAlignataIdentity( a1, 1, 8, 0 );

    m1->Add(AlignLib::makeAlignatumFromString("ABCDGHIJL"), a1);
    m1->Add(AlignLib::makeAlignatumFromString("ABCDGHIJL"), a1);

    m2->Add(AlignLib::makeAlignatumFromString("ABCDGHIJL"), a1);
    m2->Add(AlignLib::makeAlignatumFromString("ABCDGHIJL"), a1);    


    cout << *m1 << endl;
    cout << *m2 << endl;    

    AlignLib::Alignata * aa1 = AlignLib::makeAlignataVector();
    AlignLib::fillAlignataIdentity( aa1, 2, 5 );
    AlignLib::fillAlignataIdentity( aa1, 7, 7, 1 );        

    m1->Add( m2, aa1 );

    cout << *m1 << endl;
    
  }
  
  
}








